package io.github.solrudev.simpleinstaller.sampleapp.ui

import android.Manifest.permission.POST_NOTIFICATIONS
import android.content.ActivityNotFoundException
import android.content.Intent
import android.os.Build
import android.os.Bundle
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.result.contract.ActivityResultContracts.RequestPermission
import androidx.activity.result.launch
import androidx.activity.viewModels
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import io.github.solrudev.simpleinstaller.activityresult.InstallPermissionContract
import io.github.solrudev.simpleinstaller.sampleapp.databinding.ActivityMainBinding
import io.github.solrudev.simpleinstaller.sampleapp.viewmodels.MainViewModel
import kotlin.system.exitProcess

class MainActivity : AppCompatActivity() {

	private lateinit var binding: ActivityMainBinding
	private val viewModel: MainViewModel by viewModels()

	@RequiresApi(Build.VERSION_CODES.O)
	private val installPermissionLauncher = registerForActivityResult(InstallPermissionContract()) { isGranted ->
		if (isGranted) viewModel.enableInstallButton()
		requestNotificationPermission()
	}

	@RequiresApi(Build.VERSION_CODES.TIRAMISU)
	private val notificationPermissionLauncher = registerForActivityResult(RequestPermission()) { isGranted ->
		if (!isGranted) exitProcess(0)
	}

	private val pickApkLauncher = registerForActivityResult(ActivityResultContracts.GetContent()) { uri ->
		uri?.let(viewModel::installPackage)
	}

	override fun onCreate(savedInstanceState: Bundle?) {
		super.onCreate(savedInstanceState)
		binding = ActivityMainBinding.inflate(layoutInflater)
		setContentView(binding.root)
		binding.lifecycleOwner = this
		binding.activity = this
		binding.viewModel = viewModel
		if (savedInstanceState == null) {
			requestPermissions()
		}
	}

	fun onInstallButtonClick() {
		if (!viewModel.isInstalling) {
			try {
				pickApkLauncher.launch("application/vnd.android.package-archive")
			} catch (_: ActivityNotFoundException) {
			}
			return
		}
		viewModel.cancel()
	}

	fun onUninstallButtonClick() {
		val intent = Intent(this, UninstallActivity::class.java)
		startActivity(intent)
	}

	private fun requestPermissions() {
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O || packageManager.canRequestPackageInstalls()) {
			viewModel.enableInstallButton()
			requestNotificationPermission()
			return
		}
		installPermissionLauncher.launch()
	}

	private fun requestNotificationPermission() {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
			notificationPermissionLauncher.launch(POST_NOTIFICATIONS)
		}
	}
}